package net.wanji.opt.servicev2.judgeanalysis.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemCrossDayMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemCrossDay;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemGreenDay;
import net.wanji.opt.entity.judgeanalysis.CrossPoint;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemCrossDayService;
import net.wanji.opt.synthesis.pojo.CrossOptAnalysisEntity;
import net.wanji.opt.vo2.CrossOptAnalysisVO;
import net.wanji.opt.vo2.GreenwaveCrossResult;
import net.wanji.opt.vo2.JudTrunkLineCrossProblemEntityVO;
import net.wanji.opt.vo2.report.JudTrunkLineCrossProblemEntityDTO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 路口问题分析 服务实现类
 * </p>
 *
 * @author fengyi
 * @since 2025-03-13
 */
@Service
public class AnalysisProblemCrossDayServiceImpl extends ServiceImpl<AnalysisProblemCrossDayMapper, AnalysisProblemCrossDay> implements AnalysisProblemCrossDayService {

    @Resource
    private AnalysisProblemCrossDayMapper analysisProblemCrossDayMapper;


    /**
     * 路口问题分析详情
     *
     * @param id
     * @return
     */
    @Override
    public AnalysisProblemCrossDay info(Long id) {

        return getById(id);
    }

    /**
     * 路口问题分析新增
     *
     * @param param 根据需要进行传值
     * @return
     */
    @Override
    public void add(AnalysisProblemCrossDay param) {

        save(param);
    }

    /**
     * 路口问题分析修改
     *
     * @param param 根据需要进行传值
     * @return
     */
    @Override
    public void modify(AnalysisProblemCrossDay param) {

        updateById(param);
    }

    /**
     * 路口问题分析删除(单个条目)
     *
     * @param id
     * @return
     */
    @Override
    public void remove(Long id) {
        removeById(id);
    }

    /**
     * 路口问题分析删除(多个条目)
     *
     * @param ids
     * @return
     */
    @Override
    public void removes(List<Long> ids) {

        removeByIds(ids);
    }

    /**
     * 路口区域id数据添加
     *
     * @return
     */
    @Override
    public void updateCrossAreaIdByCondition() {
        //判断路口的areaId是否存在
        List<CrossPoint> crossPoints = analysisProblemCrossDayMapper.findCrossPoint();
        if (crossPoints.size() > 0) {
            //获取区域id的坐标范围
            List<CrossPoint> areaPont = analysisProblemCrossDayMapper.findAreaPoint();
            //循环判断是否存在区域内
            for (CrossPoint crossPoint : crossPoints) {
                int i = 0;
                for (CrossPoint areaPoint : areaPont) {
                    String a = crossPoint.getPoint().replaceAll(" ", ",");
                    Geometry geo = net.wanji.common.gts.GeomsConvertUtil.createPoint(a);
                    Geometry geo1 = net.wanji.common.gts.GeomsConvertUtil.genGeometry(areaPoint.getPoint());
                    if (geo.within(geo1)) {
                        //如果匹配则更新id
                        analysisProblemCrossDayMapper.updateCrossAreaIdByid(areaPoint.getId(), crossPoint.getId());
                        i = 1;
                    }
                }
                if (i == 0) {
                    //未匹配则把区域id设置为0
                    analysisProblemCrossDayMapper.updateCrossAreaIdByid("0", crossPoint.getId());
                }
            }
        }

    }

    @Override
    public List<AnalysisProblemCrossDay> getListSingleByStartAndEnd(String crossId, Integer startTime, Integer endTime) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        map.put("crossId", crossId);
        return analysisProblemCrossDayMapper.getListSingleByStartAndEnd(map);
    }

    public List<AnalysisProblemCrossDay> selectCountByType(Map<String, Object> map) {
        return analysisProblemCrossDayMapper.selectCountByType(map);
    }

    Map<String, Object> getParamMap(String crossId, Integer startTime, Integer endTime) {
        Map<String, Object> map = new HashMap();
        map.put("crossId", crossId);
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        return map;
    }

    @Override
    public Map<String, Object> selectSingerByProblemTrend(String crossId, Integer startTime, Integer endTime) throws ParseException {
        List<AnalysisProblemCrossDay> list = analysisProblemCrossDayMapper.selectSingerByProblemTrend(getParamMap(crossId, startTime, endTime));
        Date sd = DateUtil.parse(startTime.toString(), EsDateIndexUtil.YMD_FORMATTER);
        Date ed = DateUtil.parse(endTime.toString(), EsDateIndexUtil.YMD_FORMATTER);
        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis("4", DateUtil.formatDate(sd, EsDateIndexUtil.YMD_HM_FORMATTER), DateUtil.formatDate(ed, EsDateIndexUtil.YMD_HM_FORMATTER));
        List<Map<String, Object>> allList = new ArrayList<>();

        Map<String, List<AnalysisProblemCrossDay>> groupByType = list.stream().collect(Collectors.groupingBy(o -> o.getEventType(), TreeMap::new, Collectors.toList()));

        for (Map.Entry<String, List<AnalysisProblemCrossDay>> entry : groupByType.entrySet()) {
            String key = entry.getKey();
            //同一类事件每个日期问题数量
            List<AnalysisProblemCrossDay> value = entry.getValue();

            String typeName = value.get(0).getTypeName();

            // 构造一个Map用于快速查找已有数据点
            Map<String, Integer> existingDataPoints = value.stream()
                    .collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemCrossDay::getEventNumber));
            // 构造一个Map用于快速查找时长已有数据点
            Map<String, Integer> existingDate = value.stream()
                    .collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemCrossDay::getEventTotalTime));
            // 对于每个时间点，如果不存在数据则补0
            List<Integer> numberlist = sortedSet.stream()
                    .map(timePoint -> existingDataPoints.getOrDefault(timePoint, 0))
                    .collect(Collectors.toList());
            // 对于每个时间点，如果时长不存在数据则补0
            List<Integer> durationList = sortedSet.stream()
                    .map(timePoint -> existingDate.getOrDefault(timePoint, 0))
                    .collect(Collectors.toList());
            Map<String, Object> maplist = new HashMap<>();
            maplist.put("eventType", key);
            maplist.put("eventLabel", typeName);
            maplist.put("list", numberlist);
            maplist.put("durationList", durationList);
            allList.add(maplist);
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        return jsonObject;
    }

    @Override
    public CrossOptAnalysisVO getCrossOptAnalysis(String crossID, String startTime, String endTime) {
        List<CrossOptAnalysisEntity> entityList = analysisProblemCrossDayMapper.getCrossOptAnalysis(null, crossID, startTime, endTime);
        CrossOptAnalysisVO crossOptAnalysisVO = new CrossOptAnalysisVO();

        for (CrossOptAnalysisEntity c : entityList) {
            if (c.getStatus().equals("已优化")) {
                crossOptAnalysisVO.setOptimized(c.getNum());
            } else if (c.getStatus().equals("未优化")) {
                crossOptAnalysisVO.setUnoptimized(c.getNum());
            }
        }
        crossOptAnalysisVO.setTotal(crossOptAnalysisVO.getOptimized() + crossOptAnalysisVO.getUnoptimized());

        return crossOptAnalysisVO;
    }

    @Override
    public List<JudTrunkLineCrossProblemEntityVO> getTrunkLineCrossProblem(Integer greenId, Integer status, String startTime, String endTime) {
        List<GreenwaveCrossResult> greenwaveCrossResultList = analysisProblemCrossDayMapper.getTrunkLineCrossProblem(greenId, status, startTime, endTime);
        if (ObjectUtil.isEmpty(greenwaveCrossResultList)) {
            return null;
        }
        Map<Integer, List<GreenwaveCrossResult>> greenIdMap = greenwaveCrossResultList.stream().collect(Collectors.groupingBy(GreenwaveCrossResult::getGreenId));
        List<JudTrunkLineCrossProblemEntityDTO> crossProblemEntityDTOS = new ArrayList<>();
        for (List<GreenwaveCrossResult> greenwaveCrossResults : greenIdMap.values()) {
            Map<String, List<GreenwaveCrossResult>> crossIdMap = greenwaveCrossResults.stream().collect(Collectors.groupingBy(GreenwaveCrossResult::getCrossId));
            int slowCount = greenwaveCrossResults.stream().filter(x -> 2 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
            int conCount = greenwaveCrossResults.stream().filter(x -> 3 == x.getStatus() || 4 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
            for (List<GreenwaveCrossResult> crossResults : crossIdMap.values()) {
                JudTrunkLineCrossProblemEntityDTO judTrunkLineCrossProblemEntityDTO = new JudTrunkLineCrossProblemEntityDTO();
                judTrunkLineCrossProblemEntityDTO.setGreenId(crossResults.get(0).getGreenId());
                judTrunkLineCrossProblemEntityDTO.setCrossId(crossResults.get(0).getCrossId());
                judTrunkLineCrossProblemEntityDTO.setCrossName(crossResults.get(0).getName());
                if (ObjectUtil.isEmpty(status) || status == 2) {
                    int slowCountCross = crossResults.stream().filter(x -> 2 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
                    if (slowCount > 0 && slowCountCross * 100 / slowCount > 10) {
                        judTrunkLineCrossProblemEntityDTO.setCount(slowCountCross);
                        Map<Integer, Integer> dirTypeCountMap = crossResults.stream().collect(Collectors.groupingBy(
                                GreenwaveCrossResult::getDirType,
                                Collectors.summingInt(GreenwaveCrossResult::getDirCount)
                        ));
                        if(ObjectUtil.isNotEmpty(dirTypeCountMap)){
                            AtomicInteger atomicIntegerDir = new AtomicInteger(1);
                            for (Integer dirType : dirTypeCountMap.keySet()) {
                                if(atomicIntegerDir.get()>1){
                                    judTrunkLineCrossProblemEntityDTO.setOutDir(dirType);
                                    if(dirTypeCountMap.get(dirType) == 0){
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }else {
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(2);
                                    }
                                }else {
                                    judTrunkLineCrossProblemEntityDTO.setInDir(dirType);
                                    if(dirTypeCountMap.get(dirType) == 0){
                                        judTrunkLineCrossProblemEntityDTO.setStatusL(1);
                                    }else {
                                        judTrunkLineCrossProblemEntityDTO.setStatusL(2);
                                    }
                                }
                            }
                        }
                        if(ObjectUtil.isNotEmpty(crossResults.get(0))){
                            if(ObjectUtil.isNotEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                if(crossResults.get(0).getInDir() == judTrunkLineCrossProblemEntityDTO.getInDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                        judTrunkLineCrossProblemEntityDTO.setOutDir(crossResults.get(0).getOutDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }
                                if(crossResults.get(0).getOutDir() == judTrunkLineCrossProblemEntityDTO.getInDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                        judTrunkLineCrossProblemEntityDTO.setOutDir(crossResults.get(0).getInDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }
                            }
                            if(ObjectUtil.isNotEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                if(crossResults.get(0).getOutDir() == judTrunkLineCrossProblemEntityDTO.getOutDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                        judTrunkLineCrossProblemEntityDTO.setInDir(crossResults.get(0).getInDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }
                                if(crossResults.get(0).getInDir() == judTrunkLineCrossProblemEntityDTO.getOutDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                        judTrunkLineCrossProblemEntityDTO.setInDir(crossResults.get(0).getOutDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }

                            }
                        }
                    } else {
                        continue;
                    }
                }
                if (ObjectUtil.isEmpty(status) || status == 3 || status == 4) {
                    int conCountCross = crossResults.stream().filter(x -> 3 == x.getStatus() || 4 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
                    if (conCount > 0 && conCountCross * 100 / conCount > 10) {
                        if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getCount())){
                            judTrunkLineCrossProblemEntityDTO.setCount(conCountCross);
                        }
                        Map<Integer, Integer> dirTypeCountMap = crossResults.stream().collect(Collectors.groupingBy(
                                GreenwaveCrossResult::getDirType,
                                Collectors.summingInt(GreenwaveCrossResult::getDirCount)
                        ));
                        if(ObjectUtil.isNotEmpty(dirTypeCountMap)){
                            AtomicInteger atomicIntegerDir = new AtomicInteger(1);
                            for (Integer dirType : dirTypeCountMap.keySet()) {
                                if(atomicIntegerDir.get()>1){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                        judTrunkLineCrossProblemEntityDTO.setOutDir(dirType);
                                    }
                                    if(dirTypeCountMap.get(dirType) == 0){
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }else {
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(3);
                                    }
                                }else {
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                        judTrunkLineCrossProblemEntityDTO.setInDir(dirType);
                                    }
                                    if(dirTypeCountMap.get(dirType) == 0){
                                        judTrunkLineCrossProblemEntityDTO.setStatusL(1);
                                    }else {
                                        judTrunkLineCrossProblemEntityDTO.setStatusL(3);
                                    }
                                }
                            }
                        }
                        if(ObjectUtil.isNotEmpty(crossResults.get(0))){
                            if(ObjectUtil.isNotEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                if(crossResults.get(0).getInDir() == judTrunkLineCrossProblemEntityDTO.getInDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                        judTrunkLineCrossProblemEntityDTO.setOutDir(crossResults.get(0).getOutDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }
                                if(crossResults.get(0).getOutDir() == judTrunkLineCrossProblemEntityDTO.getInDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                        judTrunkLineCrossProblemEntityDTO.setOutDir(crossResults.get(0).getInDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }
                            }
                            if(ObjectUtil.isNotEmpty(judTrunkLineCrossProblemEntityDTO.getOutDir())){
                                if(crossResults.get(0).getOutDir() == judTrunkLineCrossProblemEntityDTO.getOutDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                        judTrunkLineCrossProblemEntityDTO.setInDir(crossResults.get(0).getInDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }
                                if(crossResults.get(0).getInDir() == judTrunkLineCrossProblemEntityDTO.getOutDir()){
                                    if(ObjectUtil.isEmpty(judTrunkLineCrossProblemEntityDTO.getInDir())){
                                        judTrunkLineCrossProblemEntityDTO.setInDir(crossResults.get(0).getOutDir());
                                        judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                                    }
                                }

                            }
                        }
                    } else {
                        continue;
                    }
                }
                crossProblemEntityDTOS.add(judTrunkLineCrossProblemEntityDTO);
            }

        }
        AtomicInteger atomicInteger = new AtomicInteger(1);
        crossProblemEntityDTOS.stream().sorted(Comparator.comparingInt(JudTrunkLineCrossProblemEntityDTO::getCount)).forEach(x->x.setSort(atomicInteger.getAndIncrement()));
        List<JudTrunkLineCrossProblemEntityVO> judTrunkLineCrossProblemEntityVOS = crossProblemEntityDTOS.stream().map(x -> {
            JudTrunkLineCrossProblemEntityVO judTrunkLineCrossProblemEntityVO = new JudTrunkLineCrossProblemEntityVO();
            BeanUtils.copyProperties(x, judTrunkLineCrossProblemEntityVO);
            return judTrunkLineCrossProblemEntityVO;
        }).collect(Collectors.toList());

        return judTrunkLineCrossProblemEntityVOS;
    }
}
